C     Last change:  AB   21 Aug 2008    4:21 pm

      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000)
      double precision xl(maxvar),xu(maxvar),x0(maxvar),x(maxvar)
c=======================================================
c  Input data:
c  n        - number of variables
c  nlineq   - number of linear equalities
c  neq      - number of nonlinear equalities
c  nlinineq - number of linear inequalities
c  nineq    - number of nonlinear inequalities
c=======================================================
       n  = 10
       nlineq = 0
       neq = 0
       nlinineq = 0
       nineq = 0
c=======================================================
c  lower and upper bounds for variables:
c  xl - lower bound
c  xu - upper bound
c If they are not given take xl < -10^5 and xu > 10^5
c=======================================================
      do i=1,n
        xl(i) = -1.d+08
        xu(i) = 1.d+08
      end do
      do i=1,n
        xl(i) = -32.78
        xu(i) = 32.78
      END do

c=======================================================
c  Starting point
c=======================================================
      do i=1,n
       x0(i)=1.0d+01
      END do
c=======================================================
c  Calling DFHM1
c=======================================================
      call dgm01(n,neq,nlineq,nlinineq,nineq,xl,xu,x0
     1 ,x,fvalue,convalue,cputime)

      STOP
      END

c======================================================================
c In this subroutine the user should describe the problem entering
c objective function and constraints
c======================================================================
      subroutine func(x,objf,conlineq,conlinineq,coneq,conineq)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000)
      double precision x(maxvar),coneq(maxcon),conineq(maxcon)
     1 ,conlineq(maxcon),conlinineq(maxcon)
c===============================================================
c      coneq - array of nonlinear equalities
c      conlineq - array of linear equalities
c      conlinineq - array of linear inequalities
c      conineq - array of nonlinear inequalities
c===============================================================
      pi=3.141592654d+00
      f1=0.d+00
      f2=0.d+00
      do i=1,10
       f1=f1+x(i)*x(i)
       f2=f2+dcos(2.0d+00*pi*x(i))
      END do
      objf=-2.d+01*dexp(-2.d-01*dsqrt(f1/1.0d+01))-
     1   dexp(f2/1.0d+01)+2.d+01+dexp(1.0d+00)
c=====================================================================
      return
      end

c======================================================================
c This subroutine calls both local and global solvers: OPTIMUM and
c OPTIMUM1. It also calls subroutines to solve the system of linear
c equations and identify box constraints.
c It generates output files: results.txt and local_solutions.txt
c======================================================================
      subroutine dgm01(nvar,neq,nlineq,nlinineq,nineq,xl,xu,x0
     1 ,xfinal,ob,fpenglob,cputime)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000)
      double precision x(maxvar),x3(maxvar),x0(maxvar)
     1 ,xl(maxvar),xu(maxvar),xlow(maxvar),xup(maxvar),coneq(maxcon)
     2 ,conineq(maxcon),conlineq(maxcon),conlinineq(maxcon)
     3 ,c1(maxcon,maxvar),xfinal(maxvar),x2final(maxvar)
      INTEGER l1(maxvar),l3(maxvar),indlow(maxvar),indup(maxvar)
      COMMON /cbound/xlow,xup,/csize/m,/citer/niter,/cindbound/indbound
     1 ,/cconeq/numeq,/cconin/numineq,/cpenf/fconst,/cpen/penalt
     2 ,/cl3/l3,n2,/cc1/c1,/cnshift/nshift,/cconlineq/numlineq,/cl1/l1
     3 ,/csize1/m1,/cnumneq/numneq,/cindicator/indlow,indup
     4 ,/cobstart/objstart,/cconlinineq/numlinineq
     5 ,/cnconst/nconst,/cnf/nf,/cnewpoint/newpoint
      character*30 outfi/'results.txt'/

      open(40,file=outfi)

      call cpu_time(time1)

c==========================================================
      WRITE(40,*)
      WRITE(40,112)
 112  FORMAT('     **** DISCRETE GRADIENT METHOD: VERSION 1  ****')
      WRITE(40,*)
      WRITE(40,115)
  115 FORMAT('     (c) Centre for Informatics and Applied Optimization')
      WRITE(40,*)
      WRITE(40,15)
  15  FORMAT('              (c) ITMS, University of Ballarat')
      WRITE(40,*)
      WRITE(40,*)
      WRITE(40,*)
      WRITE(40,1113)
 1113 FORMAT('       A Solver System for Nonsmooth Optimization')
      WRITE(40,*)
      WRITE(40,*)
      WRITE(40,113)
  113 FORMAT('          The author: Adil Bagirov')
      WRITE(40,*)
      WRITE(40,*)
c==========================================================
c accuracy for solution feasibility
      eps1=1.d-06
c accuracy for acceptance of solution
      eps2=1.d-04
      m=nvar
      m1=m
c box constraints
      do i=1,m
       xlow(i)=xl(i)
       xup(i)=xu(i)
      end do

      numlineq=nlineq
      numeq=neq
      numlinineq=nlinineq
      numineq=nineq

      numneq=neq
      nconst=numeq+numlinineq+numineq
      nconst1=nconst+numlineq
c nf -total number of function calls and niter - total number of iterations
      nf=0
      niter=0
c============================================================
c Printing information on input data in file results.txt
c============================================================
      WRITE(40,*)
      write(40,1)
 1    FORMAT('                          Input data')
      write(40,*)
      write(40,11) m1
  11  FORMAT('Number of variables',i50)
      WRITE(40,*)
      write(40,1112) numlineq
1112  FORMAT('Number of linear equality constraints',i32)
      WRITE(40,*)
      write(40,1114) numlinineq
1114  FORMAT('Number of linear inequality constraints',i30)
      WRITE(40,*)
      write(40,12) numeq
  12  FORMAT('Number of nonlinear equality constraints',i29)
      WRITE(40,*)
      write(40,13) numineq
  13  FORMAT('Number of nonlinear inequality constraints',i27)
      WRITE(40,*)
      WRITE(40,*)
      WRITE(40,*)
      WRITE(40,1118)
1118  FORMAT('Initial point given by user:')
      WRITE(40,*)
      do i=1,m1
       write(40,100) i,x0(i)
      end do
      WRITE(40,*) '____________________________________________________'
c=====================================================================
c Projecting onto box (if strating point is outside the box)
c=====================================================================
      fbox1=0.d+00
      do i=1,m
       fbox1=fbox1+dmax1(0.d+00,xlow(i)-x0(i))
     1  +dmax1(0.d+00,x0(i)-xup(i))
      end do
      if (fbox1.gt.0.d+00) then
        WRITE(40,*)
        WRITE(40,1014)
        WRITE(40,*)
1014    FORMAT('The initail point is outside the box'
     1          ' and it is projected onto it')
      end if
      do i=1,m
       IF(x0(i).lt.xlow(i)) x0(i)=xlow(i)
       IF(x0(i).gt.xup(i)) x0(i)=xup(i)
      end do
c=====================================================================
      call varind

      penalt=1.d+03

      call linearep
c==============================================================
      do i=1,m
       x(i)=x0(l1(i))
      end do
c==============================================================
      nshift=2
      call fv(x,f,objf)
      objstart=1.d+02*dabs(objf)
      fpen2=fconst
c===============================================================
      WRITE(40,*)
      WRITE(40,77) f
  77  FORMAT('Value of the aggregate function at initial point',f21.8)
c-----------------------------------------------------------
c  Calculations of constraint violations at initial point and their
c description in output file: results.txt
c-----------------------------------------------------------
      if (nconst1.gt.0) then
       WRITE(40,*)
       WRITE(40,2118)
2118   FORMAT('Constraint violations at initial point:')
       WRITE(40,*)
      end if

      call func(x0,objf,conlineq,conlinineq,coneq,conineq)
      vialtot=0.d+00
      if (numlineq.gt.0) then
       WRITE(40,*)
       WRITE(40,2119)
2119   FORMAT('Linear equalities:')
       WRITE(40,*)
       do i=1,numlineq
        convial=dabs(conlineq(i))
        vialtot=vialtot+convial
        write(40,101) i,convial
       end do
      end if

      if (numlinineq.gt.0) then
       WRITE(40,*)
       WRITE(40,2120)
2120   FORMAT('Linear inequalities:')
       WRITE(40,*)
       do i=1,numlinineq
        convial=dmax1(0.d+00,conlinineq(i))
        vialtot=vialtot+convial
        write(40,101) i,convial
       end do
      end if

      if (numeq.gt.0) then
       WRITE(40,*)
       WRITE(40,2121)
2121   FORMAT('Nonlinear equalities:')
       WRITE(40,*)
       do i=1,numeq
        convial=dabs(coneq(i))
        vialtot=vialtot+convial
        write(40,101) i,convial
       end do
      end if

      if (numineq.gt.0) then
       WRITE(40,*)
       WRITE(40,2122)
2122   FORMAT('Nonlinear inequalities:')
       WRITE(40,*)
       do i=1,numineq
        convial=dmax1(0.d+00,conineq(i))
        vialtot=vialtot+convial
        write(40,101) i,convial
       end do
      end if
c-----------------------------------------------------------
      IF(vialtot.gt.eps1) THEN
       write(40,*)
       write(40,*)
       write(40,122)
 122   FORMAT('The starting point is infeasible')
      END if
      IF(vialtot.le.eps1) THEN
       write(40,*)
       write(40,*)
       write(40,123)
 123   FORMAT('The strating point is feasible')
      END if
      write(40,*)
      write(40,*)
      write(40,777) vialtot
 777  FORMAT('Sum of constraint violations at initial point:',f20.8)
      write(40,*)
      write(40,*)
      WRITE(40,*) '____________________________________________________'
c=====================================================
      nshift=1
      call optimum(x)
c=====================================================
      nshift=2
      penalt=1.d+01
 102  call optimum(x)
      call fv(x,f,objf)
      if (fconst.ge.eps1) then
        penalt=1.d+01*penalt
        IF(penalt.le.2.d+04) GO TO 102
      end if
      call fv(x,f,objf)
      call restorepoint(x,x3)
c---------------------------------------------------
   7  FORMAT('Value of objective function at final point:',f20.8)
 771  FORMAT('Value of objective function:',f20.8)
 118  FORMAT('Sum of constraint violations:',f20.10)
 18   FORMAT('Variables:')
100   FORMAT('Variable',i4,f20.10)
101   FORMAT('Constraint',i4,f20.10)
c========================================================================
       WRITE(40,*)
       WRITE(40,*)
       WRITE(40,*)
       WRITE(40,*) '            FINAL SOLUTION:              '
       WRITE(40,*)
       WRITE(40,*)
       WRITE(40,*)
       write(40,7) f
       write(40,*)
       WRITE(40,*)
       WRITE(40,18)
       WRITE(40,*)
       do i=1,m1
        WRITE(40,100) i,x3(i)
       end do
       WRITE(40,*)
c-----------------------------------------------------------
c  Constraint violations at final point
c-----------------------------------------------------------
       if (nconst1.gt.0) then
        WRITE(40,*)
        WRITE(40,2123)
2123    FORMAT('Constraint violations at final point:')
        WRITE(40,*)
       end if

       call func(x3,objf,conlineq,conlinineq,coneq,conineq)
       vialtot=0.d+00
       if (numlineq.gt.0) then
        WRITE(40,*)
        WRITE(40,2124)
2124    FORMAT('Linear equalities:')
        WRITE(40,*)
        do i=1,numlineq
         convial=dabs(conlineq(i))
         vialtot=vialtot+convial
         write(40,101) i,convial
        end do
       end if

       if (numlinineq.gt.0) then
        WRITE(40,*)
        WRITE(40,2125)
2125    FORMAT('Linear inequalities:')
        WRITE(40,*)
        do i=1,numlinineq
         convial=dmax1(0.d+00,conlinineq(i))
         vialtot=vialtot+convial
         write(40,101) i,convial
        end do
       end if

       if (numeq.gt.0) then
        WRITE(40,*)
        WRITE(40,2126)
2126    FORMAT('Nonlinear equalities:')
        WRITE(40,*)
        do i=1,numeq
         convial=dabs(coneq(i))
         vialtot=vialtot+convial
         write(40,101) i,convial
        end do
       end if

       if (numineq.gt.0) then
        WRITE(40,*)
        WRITE(40,2127)
2127    FORMAT('Nonlinear inequalities:')
        WRITE(40,*)
        do i=1,numineq
         convial=dmax1(0.d+00,conineq(i))
         vialtot=vialtot+convial
         write(40,101) i,convial
        end do
       end if
       write(40,*)
       WRITE(40,118) vialtot
       WRITE(40,*)
c-----------------------------------------------------------
      WRITE(40,*)
      write(40,99) niter
 99   FORMAT('Total number of iterations',i42)
      write(40,*)
      write(40,*)
      write(40,222) nf
222   FORMAT('Total number of objective function evaluations',i22)
      write(40,*)
      write(40,*)
      write(40,*)
c      WRITE(40,1034)
c1034   FORMAT('No feasible solution has been found')
      call cpu_time(time2)
      cputime=time2-time1
      WRITE(40,1036) cputime
1036  FORMAT('Total CPU time in seconds:',f10.4)
c===========================================================
      CLOSE(40)
      return
      end

c============================================================
c Local solver: OPTIMUM
c parameters: pw - power (z(\lambda)),
c step0 - constant from Armijo algorithm
c slinit - initial value of step in the approximation subgradients
c slmin - final value of the step in the approximation subgradients
c maxiter - maximum number of iterations
c maxdisc - maximum number of appreoximated subgradients
c============================================================
      subroutine optimum(x)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000,maxit=100000)
      double precision x(maxvar),x1(maxvar),g(maxvar),v(maxvar)
     1 ,w(maxdg,maxvar),prod(maxdg,maxdg),z(maxdg),fvalues(maxit)
     2 ,gprev(maxvar)
      INTEGER ij(maxdg)
      common /csize/m,/citer/niter,/cpw/pwt,/cpenf/fconst
     1 ,/cij/ij,jvertex,/cz/z,/ckmin/kmin,/ceps4/eps4
     2 ,/cnumneq/numneq,/cfpen5/fpen5,/cfpen1/fpen1,/cnshift/nshift
c====================================================================
      pw=7.d+00
      dist1=1.d-07
      step0=-2.d-01
      div=6.d-01
      slinit=5.d+00
      eps4=1.d-05
      eps6=1.d-07
      slmin=1.d-08*slinit
      maxiter=50000
      maxdisc=MIN(m+3,200)
      sdif=1.d-05
      IF(m.le.3) mturn=4
      IF(m.gt.3) mturn=5
c====================================================================
      sl=slinit
      ncycles=dlog(slmin/slinit)/dlog(div)+1
      ncycles1=2*ncycles/3
      ncycles=ncycles+200
      call fv(x,f2,objf)
      fpen2=fconst
      pwt=1.d+00
c================================================================
      do ncycle=1,ncycles
       IF(ncycle.gt.1) sl=div*sl
       IF(sl.lt.slmin) go to 4
c----------------------------------------------------------
       IF(sl.gt.1.d-01) step0=-2.d-01
       IF((sl.le.1.d-01).AND.(sl.gt.1.d-02)) step0=-3.d-01
       IF((sl.le.1.d-02).AND.(sl.gt.1.d-04)) step0=-4.d-01
       IF(sl.le.1.d-04) step0=-5.d-01
c----------------------------------------------------------
       if (numneq.gt.0) then
        IF(ncycle.le.ncycles1) eps4=1.d-04
        IF(ncycle.gt.ncycles1) eps4=1.d-05
       end if
       IF(ncycle.gt.1) pw=9.d-01*pw
       if(pw.lt.1.5d+00) pw=1.5d+00
       t=sl**pw
       pwt=dmin1(pwt,t)
       do i=1,m
        g(i)=1.d+00/dsqrt(DBLE(m))
       end do
       niter1=niter+1
       n4=0
c================================================================
       do niter=niter1,maxiter
        n4=n4+1
        f1=f2
        fvalues(niter)=f1
        IF((nshift.eq.1).AND.(f1.lt.1.d-04)) GO TO 4
        fpen1=dmin1(1.d+01*eps4,fpen2+eps4)
c---------------------------------------------------------------
        if (n4.gt.mturn) then
         mturn2=niter-mturn+1
         ratio1=(fvalues(mturn2)-f1)/(dabs(f1)+1.d+00)
         IF(ratio1.LT.sdif) GO TO 1
        end if
        if (n4.ge.10) then
         mturn2=niter-9
         ratio1=(fvalues(mturn2)-f1)/(dabs(f1)+1.d+00)
         IF(ratio1.LT.1.d-04) GO TO 1
        end if
c--------------------------------------------------------------
        do ndg=1,maxdisc
            call dgrad(x,sl,g,v,f1,f4,ndg)
            dotprod=0.d+00
            do i=1,m
             dotprod=dotprod+v(i)*v(i)
            end do
            r=dsqrt(dotprod)
            IF(r.lt.eps6) GO TO 1
            IF(ndg.eq.1) then
                         rmean=r
                         kmin=1
                         rmin=r
            END if
            IF(ndg.gt.1) then
                         rmin=dmin1(rmin,r)
                         IF(r.eq.rmin) kmin=ndg
                         rmean=((ndg-1)*rmean+r)/ndg
            END if
            toler=dmax1(1.d-07,dist1*rmean)
            do i=1,ndg-1
             prod(ndg,i)=0.d+00
             do j=1,m
              prod(ndg,i)=prod(ndg,i)+w(i,j)*v(j)
             end do
             prod(i,ndg)=prod(ndg,i)
            end do
            prod(ndg,ndg)=dotprod
c====================================================================
            do i=1,m
             w(ndg,i)=v(i)
            end do
            call wolfe(ndg,prod)
c================================
            do i=1,m
             v(i)=0.d+00
             do j=1,jvertex
              v(i)=v(i)+w(ij(j),i)*z(j)
             END do
            END do
c================================
            r=0.d+00
            do i=1,m
             r=r+v(i)*v(i)
            end do
            r=dsqrt(r)
            if(r.lt.toler) GO TO 1
c===========================================================
             do i=1,m
              g(i)=-v(i)/r
              x1(i)=x(i)+sl*g(i)
             end do
c============================================================
             if(ndg.gt.1) then
                      rdif=0.d+00
                      do i=1,m
                       rdif=rdif+(gprev(i)-g(i))**2
                      end do
                      rdif=dsqrt(rdif)
                      if(rdif.LT.eps6) GO TO 1
                      rdif=rold-r
                      IF(rdif.LT.toler) GO TO 1
             END if
c===========================================================
             do i=1,m
              gprev(i)=g(i)
             end do
             rold=r
             call fv(x1,f4,objf)
             f3=(f4-f1)/sl
             decreas=step0*r
             if((fconst.le.fpen1).AND.(f3.lt.decreas)) then
                        fpen3=fconst
c============================================================
                        call armijo(x,g,f1,f5,f4,sl,step,r,fpen3)
                        f2=f5
                        fpen2=fpen5
                        do i=1,m
                         x(i)=x(i)+step*g(i)
                        end do
                        print 21, niter,ndg,f2,sl,fpen2
                        GO TO 2
21                      format(I6,I4,3f16.8)
             end if
         END do
c=====================================================
        go to 1
 2     END do
 1    END do
 4    return
      end

c==============================================================
c  Subroutines Wolfe and Equations solves quadratic
c  programming problem, to find
c  descent direction.
c the number of iteration is restricted by JMAX
c The vector Z contains weights of extreme points.
c The matrix A is main matrox in the system of linear equations
c from Wolfe algorithm.
c Matrix PROD contains inner products of subgradients
c===============================================================

      subroutine wolfe(ndg,prod)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000)
      common /csize/m,/w01/a,/cij/ij,jvertex,/cz/z,/ckmin/kmin
      INTEGER ij(maxdg)
      double precision z(maxdg),z1(maxdg),a(maxdg,maxdg)
     1 ,prod(maxdg,maxdg)
      j9=0
      jmax=200*ndg
      jvertex=1
      ij(1)=kmin
      z(1)=1.d+00
c=======================================
c  To calculate X
c=======================================
 1    r=0.d+00
      do i=1,jvertex
       do j=1,jvertex
        r=r+z(i)*z(j)*prod(ij(i),ij(j))
       end do
      end do
      IF(ndg.eq.1) GO TO 5
c========================================
c  To calculate <X,P_J> and J
c========================================
      t0=1.d+12
      do i=1,ndg
        t1=0.d+00
        do j=1,jvertex
          t1=t1+z(j)*prod(ij(j),i)
        end do
        if(t1.lt.t0) then
                     t0=t1
                     kmax=i
        end if
      end do
c========================================
c  First stopping criterion
c========================================
      rm=prod(kmax,kmax)
      do j=1,jvertex
       rm=dmax1(rm,prod(ij(j),ij(j)))
      end do
      r2=r-1.d-12*rm
      if(t0.gt.r2) GO TO 5
c========================================
c  Second stopping criterion
c========================================
      do i=1,jvertex
       if(kmax.eq.ij(i)) GO TO 5
      end do
c========================================
c Step 1(e) from Wolfe's algorithm
c========================================
      jvertex=jvertex+1
      ij(jvertex)=kmax
      z(jvertex)=0.d+00
c========================================
 2    do i=1,jvertex
       do j=1,jvertex
        a(i,j)=1.d+00+prod(ij(i),ij(j))
       end do
      end do
      j9=j9+1
      if(j9.gt.jmax) GO TO 5
      call equations(jvertex,z1)
      do i=1,jvertex
       if(z1(i).le.1.d-10) go to 3
      end do
      do i=1,jvertex
       z(i)=z1(i)
      end do
      go to 1
  3   teta=1.d+00
      do i=1,jvertex
       z5=z(i)-z1(i)
       if(z5.gt.1.d-10) teta=dmin1(teta,z(i)/z5)
      end do
      do i=1,jvertex
       z(i)=(1.d+00-teta)*z(i)+teta*z1(i)
       if(z(i).le.1.d-10) then
                          z(i)=0.d+00
                          kzero=i
       end if
      end do
      j2=0
      do i=1,jvertex
       IF(i.ne.kzero) then
                     j2=j2+1
                     ij(j2)=ij(i)
                     z(j2)=z(i)
       END if
      end do
      jvertex=j2
      go to 2
  5   return
      end

      subroutine equations(n,z1)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000)
      common /w01/a
      double precision a(maxdg,maxdg),z1(maxdg),b(maxdg,maxdg)
      do i=1,n
       do j=1,n
        b(i,j)=a(i,j)
       end do
       b(i,n+1)=1.d+00
      end do
      do i=1,n
       r=b(i,i)
       do j=i,n+1
        b(i,j)=b(i,j)/r
       end do
       do j=i+1,n
        do k=i+1,n+1
         b(j,k)=b(j,k)-b(i,k)*b(j,i)
        end do
       end do
      end do
      z1(n)=b(n,n+1)
      do i=1,n-1
        k=n-i
        z1(k)=b(k,n+1)
        do j=k+1,n
         z1(k)=z1(k)-b(k,j)*z1(j)
        END do
      end do
      z2=0.d+00
      do i=1,n
       z2=z2+z1(i)
      end do
      do i=1,n
       z1(i)=z1(i)/z2
      end do
      return
      end

c=====================================================================
c Subroutine dgrad calculates approximate subgradients
c=====================================================================
      subroutine dgrad(x,sl,g,dg,f1,f4,ndg)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000)
      double precision x1(maxvar),g(maxvar),x(maxvar),dg(maxvar)
      common /csize/m,/cpw/pwt,/cnshift/nshift
      a1=0.d+00
      do k=1,m
       d1=dabs(g(k))
       if (a1.lt.d1) then
         a1=d1
         imax=k
       end if
      end do
      do k=1,m
       x1(k)=x(k)+sl*g(k)
      end do
      IF(ndg.gt.1) r2=f4
      IF(ndg.eq.1) call fv(x1,r2,objf)
      flambda=r2
      dsum=0.d+00
      do k=1,m
        if(k.ne.imax) then
           r3=r2
           x1(k)=x1(k)+pwt
           call fv(x1,r2,objf)
           dg(k)=(r2-r3)/pwt
           dsum=dsum+dg(k)*sl*g(k)
        END if
      end do
      dg(imax)=(flambda-f1-dsum)/(sl*g(imax))
      return
      end

c===========================================================
c Line search (Armijo-type)
c===========================================================
      subroutine armijo(x,g,f1,f5,f4,sl,step,r,fpen3)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000)
      common /csize/m,/cpenf/fconst,/cfpen5/fpen5,/cfpen1/fpen1
     1 ,/cnstep/nstep
      double precision x(maxvar),g(maxvar),x1(maxvar),fline(1000)
     1 ,sline(1000),flpen(1000)
      k=1
      fline(k)=f4
      sline(k)=sl
      flpen(k)=fpen3
      step=sl
  1   step=step+sl
      do i=1,m
       x1(i)=x(i)+step*g(i)
      end do
      call fv(x1,f5,objf)
      nstep=nstep+1
      fpen5=fconst
      f3=f5-f1+1.d-03*step*r
      IF((fpen5.gt.fpen1).OR.(f3.gt.0.d+00)) GO TO 3
      IF((fpen5.le.fpen1).AND.(f3.le.0.d+00)) GO TO 2
 2    k=k+1
      fline(k)=f5
      sline(k)=step
      flpen(k)=fpen5
      IF(k.gt.100) GO TO 3
      GO TO 1
 3    continue
      step=sline(1)
      f5=fline(1)
      fpen5=flpen(1)
      do i=2,k
       if (f5.gt.fline(i)) then
          f5=fline(i)
          step=sline(i)
          fpen5=flpen(i)
       end if
      end do
      return
      end

c This subroutine gives values of aggregate function

      subroutine fv(x1,f,objf)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000)
      double precision x(maxvar),x1(maxvar),x2(maxvar),coneq(maxcon)
     1 ,conineq(maxcon),xlow(maxvar),xup(maxvar),c1(maxcon,maxvar)
     2 ,conlinineq(maxcon)
      INTEGER l1(maxvar),l3(maxvar),indlow(maxvar),indup(maxvar)
      COMMON /cnf/nf,/csize/m,/cconeq/numeq,/cc1/c1,/cconin/numineq
     1 ,/cpenf/fconst,/cpen/penalt,/cobstart/objstart,/cnshift/nshift
     2 ,/cbound/xlow,xup,/cl3/l3,n2,/cl1/l1,/csize1/m1
     3 ,/cindicator/indlow,indup,/cindbound/indbound
     4 ,/cconlinineq/numlinineq
c=================================================================
      do i=1,m
       x(l1(i))=x1(i)
      end do
      do i=n2,1,-1
       x(l3(i))=c1(i,m1+1)
       do j=1,m1
        x(l3(i))=x(l3(i))+c1(i,j)*x(j)
       end do
      end do
      ind1=0
c================================================================
      IF(indbound.eq.0) THEN
        do i=1,m1
         IF((x(i).lt.xlow(i)).OR.(x(i).gt.xup(i))) then
            ind1=1
            GO TO 1
         END if
        end do
      END if
   1  IF(ind1.eq.0) then
         call func(x,objf,conlineq,conlinineq,coneq,conineq)
         nf=nf+1
      END if
      IF(ind1.eq.1) then
           do i=1,m
            x2(i)=x(i)
            IF(x2(i).lt.xlow(i)) x2(i)=xlow(i)
            IF(x2(i).gt.xup(i)) x2(i)=xup(i)
           end do
         call func(x2,objf,conlineq,conlinineq,coneq,conineq)
         nf=nf+1
         objstart=objf
         call func(x,objf,conlineq,conlinineq,coneq,conineq)
         nf=nf+1
         objf=objstart
      END if
c=================================================================
      f3=0.d+00
      f4=0.d+00
      IF(indbound.eq.0) THEN
       do i=1,m
        IF((indlow(l1(i)).eq.0).AND.(indup(l1(i)).eq.0))
     1   f3=f3+dmax1(0.d+00,xlow(l1(i))-x(l1(i)))
     2    +dmax1(0.d+00,x(l1(i))-xup(l1(i)))
        IF((indlow(l1(i)).eq.1).AND.(indup(l1(i)).eq.0))
     1   f3=f3+dmax1(0.d+00,x(l1(i))-xup(l1(i)))
        IF((indlow(l1(i)).eq.0).AND.(indup(l1(i)).eq.1))
     1   f3=f3+dmax1(0.d+00,xlow(l1(i))-x(l1(i)))
       end do
       do i=1,n2
        IF((indlow(l3(i)).eq.0).AND.(indup(l3(i)).eq.0))
     1   f4=f4+dmax1(0.d+00,xlow(l3(i))-x(l3(i)))
     2    +dmax1(0.d+00,x(l3(i))-xup(l3(i)))
        IF((indlow(l3(i)).eq.1).AND.(indup(l3(i)).eq.0))
     1   f4=f4+dmax1(0.d+00,x(l3(i))-xup(l3(i)))
        IF((indlow(l3(i)).eq.0).AND.(indup(l3(i)).eq.1))
     1   f4=f4+dmax1(0.d+00,xlow(l3(i))-x(l3(i)))
       end do
      END if
      f1=0.d+00
      do i=1,numeq
       f1=f1+dmax1(0.d+00,coneq(i)-1.d-07,-coneq(i)-1.d-07)
      end do
      f2=0.d+00
      do i=1,numineq
       f2=f2+dmax1(0.d+00,conineq(i))
      end do
      f6=0.d+00
      do i=1,numlinineq
       f6=f6+dmax1(0.d+00,conlinineq(i))
      end do
      fbox=f4+f3
      f5=f1+f2
      fconst=f5+fbox+f6
      IF(nshift.eq.1) THEN
        f=f5+f6+fbox
        fconst=0.d+00
      END if
      IF(nshift.eq.2) f=objf+penalt*(f5+fbox+f6)
c=================================================================
      return
      end

c=======================================================
c This subroutine solves the system of linear equations
c and transforms the search space.
c=======================================================
      subroutine linearep
      PARAMETER(maxvar=2000, maxcon=2000)
      implicit double precision (a-h,o-z)
      DOUBLE PRECISION conlineq(maxcon),x(maxvar),x2(maxvar)
     1 ,a(maxcon,maxvar),a1(maxcon,maxvar),c1(maxcon,maxvar)
     2 ,coneq(maxcon),conineq(maxcon),conlinineq(maxcon)
      INTEGER l1(maxvar),l2(maxvar),l3(maxvar)
      COMMON /cconlineq/numlineq,/csize/m,/csize1/m1,/cl3/l3,n2
     1 ,/cl1/l1,/cc1/c1

      h=1.d-01
      do i=1,m
       x(i)=0.d+00
      end do

      n=numlineq

      do i=1,n
       do j=1,m+1
        a(i,j)=0.d+00
       end do
      end do

      do i=1,n
       call func(x,objf,conlineq,conlinineq,coneq,conineq)
       a(i,m+1)=conlineq(i)
       do j=1,m
        x(j)=h
        call func(x,objf,conlineq,conlinineq,coneq,conineq)
        a(i,j)=(conlineq(i)-a(i,m+1))/h
        x(j)=0.d+00
       end do
      end do

      do i=1,n
       do j=1,m+1
        a1(i,j)=a(i,j)
       end do
      end do

      n1=0
      n2=0

      do j=1,m
       do i=1,n
        do k=1,n2
         IF(i.eq.l2(k)) GO TO 4
        end do
        IF(dabs(a1(i,j)).ge.1.d-10) GO TO 1
  4    end do
       n1=n1+1
       l1(n1)=j
       GO TO 2
  1    n2=n2+1
       l2(n2)=i
       l3(n2)=j
       ncurrent=i
       r=a1(ncurrent,j)
       do k1=1,m+1
        a1(ncurrent,k1)=a1(ncurrent,k1)/r
       end do
       dpen0=0.d+00
       do k1=1,m+1
        c1(n2,k1)=-a1(ncurrent,k1)
       end do
       c1(n2,j)=0.d+00

       do i=1,n
        do k1=1,n2
         IF(i.eq.l2(k1)) GO TO 5
        end do
        b1=a1(i,j)
        do k1=1,m+1
         a1(i,k1)=a1(i,k1)-b1*a1(ncurrent,k1)
        end do
  5    end do

 2    end do
      m1=m
      m=n1

      return
      end

c===============================================================
c This subroutine makes reverse transforamtion returning points to
c the original space.
c===============================================================

      subroutine restorepoint(x1,x)
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000, maxcon=2000, maxdg=1000)
      double precision x(maxvar),x1(maxvar),c1(maxcon,maxvar)
      INTEGER l1(maxvar),l3(maxvar)
      common /csize/m,/cl3/l3,n2,/cl1/l1,/csize1/m1,/cc1/c1
c=================================================================
      do i=1,m
       x(l1(i))=x1(i)
      end do
      do i=n2,1,-1
       x(l3(i))=c1(i,m1+1)
       do j=1,m1
        x(l3(i))=x(l3(i))+c1(i,j)*x(j)
       end do
      end do
      return
      end
c=============================================================
c This subroutine indentifies variables with box constraints
c=============================================================
      subroutine varind
      implicit double precision (a-h,o-z)
      PARAMETER(maxvar=2000)
      double precision xlow(maxvar),xup(maxvar)
      INTEGER indlow(maxvar),indup(maxvar)
      COMMON /csize1/m,/cbound/xlow,xup,/cindicator/indlow,indup
     1 ,/cindbound/indbound
      do i=1,m
       IF(xlow(i).LT.-1.d+05) indlow(i)=1
       IF(xlow(i).GE.-1.d+05) indlow(i)=0
       IF(xup(i).gt.1.d+05) indup(i)=1
       IF(xup(i).le.1.d+05) indup(i)=0
      end do
      indbound=1
      do i=1,m
       IF((indlow(i).eq.0).or.(indup(i).eq.0)) THEN
         indbound=0
         GO TO 1
       END if
      end do
  1   RETURN
      END
